@@ -2,7 +2,7 @@
   packageStartupMessage("")
   packageStartupMessage("***********************************************************")
   packageStartupMessage("")
-  packageStartupMessage("      This is 'SOMbrero' package, v 1.1")
+  packageStartupMessage("      This is 'SOMbrero' package, v 1.2")
   packageStartupMessage("")
   packageStartupMessage("Citation details with citation('SOMbrero')")
   packageStartupMessage("")
@@ -10,7 +10,7 @@
   packageStartupMessage("")
   packageStartupMessage("Use sombreroGUI() to start the Graphical Interface.")
   packageStartupMessage("")
-  packageStartupMessage("Warning! This package has been implemented by girls.")
+  packageStartupMessage("Warning! This package has been implemented (mostly) by girls.")
   packageStartupMessage("Default colors may not be suited for men.")
   packageStartupMessage("")
   packageStartupMessage("***********************************************************")
